from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import sys
import time

from datetime import datetime

import sys

sys.path.append("../")
from utils import datetimeUtil


class SimpleOHLCV:

    def __init__(self, o=[], h=[], l=[], c=[], v=[], ts=[], init=False):
        self.open = []
        self.high = []
        self.low = []
        self.close = []
        self.volume = []
        self.ts = []

        if init:
            self.open.append(0)
            self.high.append(0)
            self.low.append(0)
            self.close.append(0)
            self.volume.append(0)
            self.ts.append(datetimeUtil.ts2String(int(time.time())))

        if (len(o)):
            self.open.extend(o)
        if (len(h)):
            self.high.extend(h)
        if (len(l)):
            self.low.extend(l)
        if (len(c)):
            self.close.extend(c)
        if (len(v)):
            self.volume.extend(v)
        if (len(ts)):
            self.ts.extend(ts)

    def lastTs(self, format=0):
        '''
        @return unix时间戳，单位，秒
        '''
        ret = 0
        if self.len() > 0:
            ret = datetimeUtil.string2TsSec(self.ts[-1])

        if 1 == format:
            ret = datetimeUtil.ts2String(ret)

        return ret

    def len(self):
        print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "DEBUG", 'len',
              len(self.open), len(self.high), len(self.low), len(self.close),
              len(self.volume), len(self.ts))

        # 长度判断
        #   其中的 exit（）后续应该要屏蔽
        if len(self.open) != len(self.high) or len(self.open) != len(
                self.low) or len(self.open) != len(self.close) or len(
                    self.open) != len(self.volume) or len(self.open) != len(
                        self.ts):
            print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "ERROR",
                  'lenError', len(self.open), len(self.high), len(self.low),
                  len(self.close), len(self.volume), len(self.ts))
            exit()

        return len(self.open)

    def extend(self, input=None):
        strMaxTs = datetimeUtil.ts2String(0)
        if self.len() > 0:
            strMaxTs = self.ts[-1]

        maxTs = datetimeUtil.string2TsSec(strMaxTs)

        inputAfterFilter = input.tsFilter(strTs=strMaxTs)

        if inputAfterFilter.len() <= 0:
            print(
                'extend, inputAfterFilter is null',
                'strMaxTs',
                strMaxTs,
                'maxTs',
                maxTs,
            )
            return

        print('extend', 'len inputAfterFilter', inputAfterFilter.len())

        self.open.extend(inputAfterFilter.open)
        self.high.extend(inputAfterFilter.high)
        self.low.extend(inputAfterFilter.low)
        self.close.extend(inputAfterFilter.close)
        self.volume.extend(inputAfterFilter.volume)
        self.ts.extend(inputAfterFilter.ts)

    def dumps(self, index0=None, index1=None):
        return {
            'open': self.open[index0:index1],
            'high': self.high[index0:index1],
            'low': self.low[index0:index1],
            'close': self.close[index0:index1],
            'volume': self.volume[index0:index1],
            'ts': self.ts[index0:index1]
        }

    def delItems(self, index0=None, index1=None):
        del self.open[index0:index1]
        del self.high[index0:index1]
        del self.low[index0:index1]
        del self.close[index0:index1]
        del self.volume[index0:index1]
        del self.ts[index0:index1]

    def tsFilter(self, strTs=None):
        ''''
        @brief 根据时间戳过滤数据
               把小于ts的数据，都过滤掉
        '''
        i = 0
        for currTs in self.ts:
            if datetimeUtil.string2TsSec(currTs) > datetimeUtil.string2TsSec(
                    strTs):
                break
            i += 1

        return self.subset(index0=i)

    def subset(self, index0=None, index1=None):
        retOhlcv = SimpleOHLCV()

        retOhlcv.open = self.open[index0:index1]
        retOhlcv.high = self.high[index0:index1]
        retOhlcv.low = self.low[index0:index1]
        retOhlcv.close = self.close[index0:index1]
        retOhlcv.volume = self.volume[index0:index1]
        retOhlcv.ts = self.ts[index0:index1]

        return retOhlcv

    def clone(self):
        ret = SimpleOHLCV()

        ret.extend(self)

        return ret

    @staticmethod
    def convertFromDatas(feed=None, size=1000):
        '''
        @brief backtrader 的 datas转换
        '''
        # ret = SimpleOHLCV()

        arrDateTime = feed.datetime.get(size=size)
        arrTs = []
        for currDateTime in arrDateTime:
            # print(currDateTime, currDateTime)
            currTs = datetimeUtil.datasDateTime2TsSec(currDateTime)
            arrTs.append(currTs)

        ret = SimpleOHLCV(o=feed.open.get(size=size).tolist(),
                          h=feed.high.get(size=size).tolist(),
                          l=feed.low.get(size=size).tolist(),
                          c=feed.close.get(size=size).tolist(),
                          v=feed.volume.get(size=size).tolist(),
                          ts=arrTs)

        return ret



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import os


def proEnv():
    return 'pro' == instPyEnv.pyEnv


def devEnv():
    return 'dev' == instPyEnv.pyEnv


class PyEnv():

    _instance = None

    def __new__(cls):
        if cls._instance is None:
            # print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "DEBUG",
            #       "Creating the object")
            cls._instance = super(PyEnv, cls).__new__(cls)
            # Put any initialization here.
        return cls._instance

    def __init__(self):
        self.pyEnv = os.environ.get("PYTHON_ENV")
        print('pyEnv', self.pyEnv)


instPyEnv = PyEnv()


from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import backtrader as bt
from datetime import datetime
import math


class MathLog(bt.Indicator):
    '''
    '''
    lines = ('localline', )

    def next(self):
        self.line[0] = math.log(self.data[0])


def MathLogFunc(data=[]):
    arrRet = []
    for currItem in data:
        arrRet.append(math.log(currItem))

    return arrRet


def MathExpFunc(data=[]):
    arrRet = []
    for currItem in data:
        arrRet.append(math.exp(currItem))

    return arrRet



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from datetime import datetime
from os import times
import pytz
import time
import backtrader as bt


def ts2String(timestamp, add8Hours=False, dateFormat=0):
    '''
    @brief 时间戳转北京时间 字符串
            把 timestamp 时间戳（秒/毫秒） 转为字符串
            timestamp 如果是毫秒，内部先转为秒，然后再转为字符串  YYYYMMDD HH:Mi:SS
    @param  timestamp 时间戳； 单位，秒 或者毫秒；
    @param  add8Hours 是否增加8小时，用于做兼容； True，考虑，在 timestamp加8小时；  False，不考虑，直接使用原  timestamp
    '''
    tsInSec = int(timestamp)

    strFormat = '%Y%m%d %H:%M:%S'

    if 1 == dateFormat:
        strFormat = '%Y-%m-%d %H:%M:%S'

    # 如果是毫秒格式，则转化为秒
    if tsInSec > 1000000000000:
        tsInSec = tsInSec / 1000

    if add8Hours:
        tsInSec += 3600 * 8

    tz = pytz.timezone('Asia/Shanghai')  #东八区
    return datetime.fromtimestamp(int(tsInSec), tz).strftime(strFormat)


def string2TsSec(strSec, dateFormat=0):
    '''
    @brief "YYYYMMDD HH:Mi:SS" 转换为 时间戳（秒）
    '''
    if 1 == dateFormat:
        structTime = time.strptime(strSec, "%Y-%m-%d %H:%M:%S")
    elif 2 == dateFormat:
        structTime = time.strptime(strSec, "%Y/%m/%d %H:%M")
    else:
        structTime = time.strptime(strSec, "%Y%m%d %H:%M:%S")

    timestamp = time.mktime(structTime)
    return int(timestamp)


def string2TsMs(strMs):
    '''
    @brief "YYYYMMDD HH:Mi:SS.ms" 转换为 时间戳（毫秒）
    '''

    structTime = datetime.strptime(strMs, "%Y%m%d %H:%M:%S.%f")
    timestamp = time.mktime(
        structTime.timetuple()) * 1000.0 + structTime.microsecond / 1000.0
    return int(timestamp)


def string2Datetime(strMs):
    '''
    @brief "YYYYMMDD HH:Mi:SS" 转换为 datetime
    '''
    return datetime.strptime(strMs, "%Y%m%d %H:%M:%S")


def datasDateTime2TsSec(datasDateTime):
    '''
    @brief bt.strategy self.datas[N].datetime 转换成 时间戳（秒）
          20221125 发现 datas[N].datetime 会把交易所的k线时间戳 ， 减掉 8小时，然后转换成datas[N].datetime 保存；
          调整 backtrader 时区，可能可以解决  datas[N].datetime问题；
          
          但应用跑了这么久，调整 backtrader 时区是否会矫枉过正 造成其他相关模块的时间问题，不得而知；
          因此考虑单独解析这个 datas[N].datetime，需要自行增加 8小时
    '''
    return ts2String(bt.num2date(datasDateTime).timestamp(), add8Hours=True)


from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import unittest
import sys

from datetimeUtil import *


class TestCase(unittest.TestCase):
    tsInMs = 1668222660000
    str0 = ts2String(tsInMs)
    print('tsInMs', tsInMs, 'str0', str0)

    strSec = "20201005 10:12:56"
    print('strSec', strSec, 'string2TsSec(strSec)', string2TsSec(strSec))

    strMs = '20190114 15:22:18.123'
    print('strMs', strMs, 'string2TsMs(strMs)', string2TsMs(strMs))

    tsInSec = string2TsSec(str0)
    print('str0', str0, 'tsInSec', tsInSec)
    assert tsInMs / 1000 == tsInSec

    tsInSec = 1669345200
    str1 = ts2String(tsInSec)
    str2 = ts2String(tsInSec, add8Hours=True, dateFormat=1)
    print('tsInSec', tsInSec, 'str1', str1, 'str2', str2)

    datasDateTime = 738484.4583333334

    str3 = datasDateTime2TsSec(datasDateTime)
    print('datasDateTime', datasDateTime, 'str3', str3)

    print('time()', time.time())

    tsInMs = 0
    str0 = ts2String(tsInMs)
    print('tsInMs', tsInMs, 'str0', str0)


if __name__ == 'main':
    unittest.main()



import unittest
# import OHLCV
import sys
import json

# import Test

sys.path.append("../")
from feed.SimpleOHLCV import SimpleOHLCV


class TestCase(unittest.TestCase):
    ohlcv = SimpleOHLCV()

    print('ohlcv.len', ohlcv.len(), 'ohlcv dumps', ohlcv.dumps())

    ohlcv = SimpleOHLCV(
        o=[11, 12, 13],
        h=[21, 22, 23],
        l=[31, 32, 33],
        c=[41, 42, 43],
        v=[51, 52, 53],
        ts=['20201229 00:01:00', '20201229 00:02:00', '20201229 00:03:00'])

    print('ohlcv.len', ohlcv.len(), 'ohlcv dumps', ohlcv.dumps())

    ohlcv1 = SimpleOHLCV(
        o=[14, 15, 16],
        h=[24, 25, 26],
        l=[34, 35, 36],
        c=[44, 45, 46],
        v=[54, 55, 56],
        ts=['20201229 00:04:00', '20201229 00:05:00', '20201229 00:06:00'])

    ohlcv.extend(ohlcv1)

    print('ohlcv.len', ohlcv.len(), 'ohlcv dumps', ohlcv.dumps())

    print('lastTs', ohlcv1.lastTs(format=1))

    # print(ohlcv.open.get(size=5))


#     ohlcv.delItems(-1, None)

#     print('after delItems, ohlcv.len', ohlcv.len(), 'ohlcv dumps',
#           ohlcv.dumps())

#     subOhlcv0 = ohlcv.subset(0, -1)
#     print('subOhlcv0.len', subOhlcv0.len(), 'subOhlcv0 dumps',
#           subOhlcv0.dumps())

#     tmpOhlcv = SimpleOHLCV(
#         o=[14, 15, 16],
#         h=[24, 25, 26],
#         l=[34, 35, 36],
#         c=[44, 45, 46],
#         v=[54, 55, 56],
#         ts=['20201229 00:01:00', '20201229 00:02:00', '20201229 00:03:00'])

#     print('tmpOhlcv', tmpOhlcv.dumps())

#     tmpOhlcv1 = tmpOhlcv.tsFilter(strTs='20201229 00:01:00')

#     print('tmpOhlcv1', tmpOhlcv1.dumps())

if __name__ == 'main':
    unittest.main()



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
import sys
from pyrsistent import freeze

sys.path.append("../")
from infra.pyEnv import env

# 币对
Symbol = freeze({
    'BTC/USDT:USDT': 'BTC/USDT:USDT',
    'ETH/USDT:USDT': 'ETH/USDT:USDT',
    'XRP/USDT:USDT': 'XRP/USDT:USDT',
    'OP/USDT:USDT': 'OP/USDT:USDT',
    'OKT/USDT:USDT': 'OKT/USDT:USDT',
    'DYDX/USDT:USDT': 'DYDX/USDT:USDT',
    'GMT/USDT:USDT': 'GMT/USDT:USDT',
})

# 合约张数
ContractSize = None
if env.proEnv():
    ContractSize = freeze({
        Symbol['BTC/USDT:USDT']: 0.01,
        Symbol['ETH/USDT:USDT']: 0.1,
        Symbol['XRP/USDT:USDT']: 100,
        Symbol['OP/USDT:USDT']: 1,
        Symbol['OKT/USDT:USDT']: 0,
        Symbol['DYDX/USDT:USDT']: 1,
        Symbol['GMT/USDT:USDT']: 1,
    })
else:
    ContractSize = freeze({
        Symbol['BTC/USDT:USDT']: 0.001,
        Symbol['ETH/USDT:USDT']: 0.01,
        Symbol['XRP/USDT:USDT']: 100,
        Symbol['OP/USDT:USDT']: 1,
        Symbol['OKT/USDT:USDT']: 0,
        Symbol['DYDX/USDT:USDT']: 0,
        Symbol['GMT/USDT:USDT']: 0.1
    })

# 产品ID
InstId = freeze({
    Symbol['BTC/USDT:USDT']: 'BTC-USDT-SWAP',
    Symbol['ETH/USDT:USDT']: 'ETH-USDT-SWAP',
    Symbol['XRP/USDT:USDT']: 'XRP-USDT-SWAP',
    Symbol['OP/USDT:USDT']: 'OP-USDT-SWAP',
    Symbol['OKT/USDT:USDT']: 'OKT-USDT-SWAP',
    Symbol['DYDX/USDT:USDT']: 'DYDX-USDT-SWAP',
    Symbol['GMT/USDT:USDT']: 'GMT-USDT-SWAP',
})

# 持仓方向
PosSide = freeze({'LONG': 'long', 'SHORT': 'short'})

# 止盈止损 订单类型
SltpOrderType = freeze({
    'conditional': 'conditional',  # 单向止盈止损
    'oco': 'oco',  # 双向止盈止损
})

# 交易模式
TdMode = freeze({
    'isolated': 'isolated',  # 逐仓
    'cross': 'cross'  # 全仓
})

# 订单方向
Side = freeze({'buy': 'buy', 'sell': 'sell'})

# 时间（以毫秒为单位)
MSTIME_MS = freeze({'value': 1})
MSTIME_S = freeze({'value': 1000 * MSTIME_MS['value']})
MSTIME_MI = freeze({'value': 60 * MSTIME_S['value']})
MSTIME_H = freeze({'value': 3600 * MSTIME_S['value']})
MSTIME_D = freeze({'value': 24 * MSTIME_H['value']})
MSTIME_W = freeze({'value': 7 * MSTIME_D['value']})
MSTIME_MO = freeze({'value': 30 * MSTIME_D['value']})
MSTIME_Y = freeze({'value': 365 * MSTIME_D['value']})

# 交易所缩写
Exchange = freeze({'OKX': 'OKX', 'Binance': 'Binance'})

# 交易所Api参数接口信息
ExangeApiParamInfo = freeze({
    # exchange ID
    'exId': {
        'OKX': 1
    },

    # k线历史查询单页最大记录个数s
    'ohlcvHisPageLimit': {
        'OKX': 100
    },
})



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import backtrader as bt
from datetime import datetime
import math
import numpy
import pandas as pd
import sys
import time

sys.path.append("../")
from utils import datetimeUtil
from indicator.heikin_ashi import HA
from feed.SimpleOHLCV import SimpleOHLCV


class LastHourAvgInd(bt.Indicator):
    '''
    @brief 计算一个小时内的平均值
            该指标当前未考虑通用型，暂时只能用在 1m 周期上
    '''
    lines = ('localline', )

    def next(self):
        # 需要计算的柱子数
        #   按当前的分钟数来计算，综合考虑当前已有的柱子数（len(self.data.datetime)）

        lastHourCandleNum = 60  # 默认是60根柱子

        # 第0~59分的赋值
        if datetime.now().minute > 0:
            lastHourCandleNum = min(datetime.now().minute,
                                    len(self.data.datetime))

        retAvg = math.fsum(
            self.data.get(size=lastHourCandleNum)) / lastHourCandleNum

        print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "DEBUG",
              'LastHourAvgInd', 'lastHourCandleNum', lastHourCandleNum,
              'retAvg', retAvg)

        self.line[0] = retAvg


class LastNAvg(bt.indicators.PeriodN):
    '''
    @brief 计算 N个柱子内的平均值
    '''
    lines = ('localline', )

    def next(self):

        print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "DEBUG",
              'LastNAvg next() 000000', 'period', self.p.period)

        if (self.p.period <= 0):
            return

        print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "DEBUG",
              'LastNAvg next() 111111', 'self.data.get(size=self.p.period)',
              self.data.get(size=self.p.period))

        retAvg = math.fsum(self.data.get(size=self.p.period)) / self.p.period

        print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "DEBUG",
              'LastNAvg next()', 'period', self.p.period, 'retAvg', retAvg)

        self.line[0] = retAvg


def ConvertDateTime(feed=None, size=1000):
    arrDateTime = feed.datetime.get(size=size)
    arrTs = []
    for currDateTime in arrDateTime:
        # print(currDateTime, currDateTime)
        currTs = datetimeUtil.datasDateTime2TsSec(currDateTime)
        arrTs.append(currTs)

    return {
        'open': feed.open.get(size=size),
        'high': feed.high.get(size=size),
        'low': feed.low.get(size=size),
        'close': feed.close.get(size=size),
        'volume': feed.volume.get(size=size),
        'ts': arrTs
    }


def appendOhlcv(src=None, appendItem=None):
    print(
        'appendOhlcv',
        # 'src', src,
        'typeOf(src[open])',
        type(src['open']),
        #   'appendItem',
        #       appendItem
    )
    # print('src[open] len', len(src['open']))

    src['open'].append(appendItem['open'])
    src['high'].append(appendItem['high'])
    src['low'].append(appendItem['low'])
    src['close'].append(appendItem['close'])
    src['volume'].append(appendItem['volume'])
    src['ts'].append(appendItem['ts'])

    return src


def OhlcvHA(ohlcv=None):
    return HA(o=ohlcv.open,
              h=ohlcv.high,
              l=ohlcv.low,
              c=ohlcv.close,
              v=ohlcv.volume,
              ts=ohlcv.ts)


def FeedHA(feed=None, size=1000, inputResConvert=None):
    resConvert = None
    if (None != feed):
        resConvert = ConvertDateTime(feed, size)
    else:
        resConvert = inputResConvert

    return HA(o=resConvert['open'],
              h=resConvert['high'],
              l=resConvert['low'],
              c=resConvert['close'],
              v=resConvert['volume'],
              ts=resConvert['ts'])


def OhlcvAvg(ohlcv=None):
    retAvg = SimpleOHLCV(init=True)

    size = ohlcv.len()
    if ohlcv.len() <= 0:
        return retAvg

    print(
        # 'dataFeed', dataFeed,
        'FeedAvg',
        'size',
        size)

    for i in range(size):
        retAvg.open[0] += ohlcv.open[i]
        retAvg.high[0] += ohlcv.high[i]
        retAvg.low[0] += ohlcv.low[i]
        retAvg.close[0] += ohlcv.close[i]
        retAvg.volume[0] += ohlcv.volume[i]

    retAvg.open[0] = retAvg.open[0] / size
    retAvg.high[0] = retAvg.high[0] / size
    retAvg.low[0] = retAvg.low[0] / size
    retAvg.close[0] = retAvg.close[0] / size
    retAvg.volume[0] = retAvg.volume[0] / size

    return retAvg


def FeedAvg(localFeed=None, size=1000):
    retAvg = {
        'open': 0,
        'high': 0,
        'low': 0,
        'close': 0,
        'volume': 0,
        'ts': datetimeUtil.ts2String(int(time.time()))
    }

    if size <= 0:
        return retAvg

    dataFeed = {
        'open': localFeed['open'][-1 * size:],
        'high': localFeed['high'][-1 * size:],
        'low': localFeed['low'][-1 * size:],
        'close': localFeed['close'][-1 * size:],
        'volume': localFeed['volume'][-1 * size:]
    }

    print(
        # 'dataFeed', dataFeed,
        'FeedAvg',
        'size',
        size)

    for i in range(size):
        retAvg['open'] += dataFeed['open'][i]
        retAvg['high'] += dataFeed['high'][i]
        retAvg['low'] += dataFeed['low'][i]
        retAvg['close'] += dataFeed['close'][i]
        retAvg['volume'] += dataFeed['volume'][i]

    retAvg['open'] = retAvg['open'] / size
    retAvg['high'] = retAvg['high'] / size
    retAvg['low'] = retAvg['low'] / size
    retAvg['close'] = retAvg['close'] / size
    retAvg['volume'] = retAvg['volume'] / size

    return retAvg



import numpy as np
from talib import abstract
import math
from datetime import datetime

import sys

sys.path.append("../")

SMA = abstract.SMA
EMA = abstract.EMA

def BiasFunc(arrClose=[], period0=20, period1=60):
    '''
    @brief
    @return retCode 返回值，0 ，成功； 非0 ，失败；
    '''

    inputs = {
        'close': np.array(arrClose),
    }

    retCode = 4000
    retBias = []

    # # 规避柱子数不足，导致taLib的报错
    # #   这里的取值，跟以下 mode中的计算有关(".../2"、".../3")
    if period0 < 5 or period1 < 5:
        print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "ERROR",
              'param error in BiasFunc', 'period0', period0, 'period1',
              period1)
        return retCode, retBias

    ma0 = EMA(inputs, int(period0))
    ma1 = EMA(inputs, int(period1))

    retBias = (ma0 - ma1) / ma0 * 1000
    #                   WMA(inputs, timeperiod=period),
    #                   timeperiod=int(round(math.sqrt(period))))

    # if 0 == mode:
    #     retHull = WMA(2 * WMA(inputs, timeperiod=int(period / 2)) -
    #                   WMA(inputs, timeperiod=period),
    #                   timeperiod=int(round(math.sqrt(period))))
    # elif 1 == mode:
    #     retHull = EMA(2 * EMA(inputs, timeperiod=int(period / 2)) -
    #                   EMA(inputs, timeperiod=period),
    #                   timeperiod=int(round(math.sqrt(period))))
    # elif 2 == mode:
    #     period = int(period / 2)
    #     retHull = WMA(WMA(inputs, timeperiod=int(period / 3)) * 3 -
    #                   WMA(inputs, timeperiod=int(period / 2)) -
    #                   WMA(inputs, timeperiod=period),
    #                   timeperiod=period)

    # hisNumber = 2
    # lookupNum = hisNumber + 1
    # if len(retHull) >= lookupNum and False == math.isnan(
    #         retHull[-1]) and False == math.isnan(retHull[-3]):

    #     if retHull[-1] > retHull[-3]:
    #         retColor = HullColor['green']
    #     else:
    #         retColor = HullColor['red']

    # arr = np.array([3.14], dtype=np.float64)

    retCode = 0
    return retCode, retBias.tolist()



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from pyrsistent import freeze

import sys
import backtrader as bt

sys.path.append("../")
from common.enums import MSTIME_MI, MSTIME_H, MSTIME_D, MSTIME_W, MSTIME_Y

# , , ,
#      , , , , NoTimeFrame
# k线周期的单位
TimeframeUni = freeze({
    'ms': bt.TimeFrame.MicroSeconds,
    's': bt.TimeFrame.Seconds,
    'mi': bt.TimeFrame.Minutes,
    'd': bt.TimeFrame.Days,
    'w': bt.TimeFrame.Weeks,
    'mo': bt.TimeFrame.Months,
    'y': bt.TimeFrame.Years
})

# k线周期
Timeframe = freeze({
    '1m': {
        'name': '1m',
        'exBar': '1m',
        'dbFieldValue': '1m',
        'msTime': 1 * MSTIME_MI['value']
    },
    '3m': {
        'name': '3m',
        'exBar': '3m',
        'dbFieldValue': '3m',
        'msTime': 3 * MSTIME_MI['value']
    },
    '5m': {
        'name': '5m',
        'exBar': '5m',
        'dbFieldValue': '5m',
        'msTime': 5 * MSTIME_MI['value']
    },
    '10m': {
        'name': '10m',
        'exBar': '10m',
        'dbFieldValue': '10m',
        'msTime': 10 * MSTIME_MI['value']
    },
    '15m': {
        'name': '15m',
        'exBar': '15m',
        'dbFieldValue': '15m',
        'msTime': 15 * MSTIME_MI['value']
    },
    '30m': {
        'name': '30m',
        'exBar': '30m',
        'dbFieldValue': '30m',
        'msTime': 30 * MSTIME_MI['value']
    },
    '1h': {
        'name': '1h',
        'exBar': '1H',
        'dbFieldValue': '1h',
        'msTime': 1 * MSTIME_H['value']
    },
    '90m': {
        'name': '90m',
        'exBar': '90m',
        'msTime': 1.5 * MSTIME_H['value']
    },
    '2h': {
        'name': '2h',
        'exBar': '2H',
        'msTime': 2 * MSTIME_H['value']
    },
    '3h': {
        'name': '3h',
        'exBar': '3H',
        'msTime': 3 * MSTIME_H['value']
    },
    '4h': {
        'name': '4h',
        'exBar': '4H',
        'dbFieldValue': '4h',
        'msTime': 4 * MSTIME_H['value']
    },
    '6h': {
        'name': '6h',
        'exBar': '6H',
        'msTime': 6 * MSTIME_H['value']
    },
    '8h': {
        'name': '8h',
        'exBar': '8H',
        'msTime': 8 * MSTIME_H['value']
    },
    '12h': {
        'name': '12h',
        'exBar': '12H',
        'msTime': 12 * MSTIME_H['value']
    },
    '1d': {
        'name': '1d',
        'exBar': '1D',
        'msTime': 1 * MSTIME_D['value']
    },
    '3d': {
        'name': '3d',
        'exBar': '3D',
        'msTime': 3 * MSTIME_D['value']
    },
    '1w': {
        'name': '1w',
        'exBar': '1W',
        'msTime': 1 * MSTIME_W['value']
    },
    '2w': {
        'name': '2w',
        'exBar': '2W',
        'msTime': 2 * MSTIME_W['value']
    },
    '1M': {
        'name': '1M',
        'exBar': '1M',
        'msTime': 1 * MSTIME_MI['value']
    },
    '3M': {
        'name': '3M',
        'exBar': '3M',
        'msTime': 3 * MSTIME_MI['value']
    },
    '6M': {
        'name': '6M',
        'exBar': '6M',
        'msTime': 6 * MSTIME_MI['value']
    },
    '1y': {
        'name': '1y',
        'exBar': '1Y',
        'msTime': 1 * MSTIME_Y['value']
    }
})



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
from datetime import datetime
import pandas as pd


def HA(o=[], h=[], l=[], c=[], v=[], ts=[]):
    ha = None
    try:
        inputs = {
            'open': o,
            'high': h,
            'low': l,
            'close': c,
            'volume': v,
            'ts': ts
        }

        df = pd.DataFrame(
            data=inputs,
            columns=['open', 'high', 'low', 'close', 'volume', 'ts'])
        df = df.set_index('ts')

        ha = _heikin_ashi(df)
    except Exception as e:
        print(datetime.now().strftime("%Y%m%d %H:%M:%S"), "ERROR",
              "_heikin_ashi ", 'Exception', str(e))

    return ha


def _heikin_ashi(df):
    '''
    @brief  heikin ashi
            参考 https://github.com/songv1/2019-Cryptocurrency-Trading-Algorithm/blob/master/crypto_heikinashi_charts.py
    '''
    heikin_ashi_df = pd.DataFrame(index=df.index.values,
                                  columns=['open', 'high', 'low', 'close'])
    heikin_ashi_df['close'] = (df['open'] + df['high'] + df['low'] +
                               df['close']) / 4
    for i in range(len(df)):
        if i == 0:
            heikin_ashi_df.iat[0, 0] = df['open'].iloc[0]
        else:
            heikin_ashi_df.iat[i, 0] = (heikin_ashi_df.iat[i - 1, 0] +
                                        heikin_ashi_df.iat[i - 1, 3]) / 2
    heikin_ashi_df['high'] = heikin_ashi_df.loc[:, ['open', 'close']].join(
        df['high']).max(axis=1)
    heikin_ashi_df['low'] = heikin_ashi_df.loc[:, ['open', 'close']].join(
        df['low']).min(axis=1)
    return heikin_ashi_df



from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import sys

sys.path.append("../../")

from nginqcomm.common import enums

sys.path.append("../")
from utils import maths

# @brief 报价币的数量转换为张数（size）
#        如 币对 BTC-USDT-SWAP  中的 U数量转换为 BTC的张数(size)


def quoteAmountToSize(symbol=None, qAmount=0, close=0):
    retSize = 0
    contractSize = enums.ContractSize.get(symbol)
    if 0 != close and None != contractSize:
        retSize = qAmount / close / contractSize
    return retSize


def sizeToQuoteAmount(symbol=None, size=0, close=0):
    retQuoteAmount = 0
    contractSize = enums.ContractSize.get(symbol)
    if 0 != close and None != contractSize:
        retQuoteAmount = size * close * contractSize

    return retQuoteAmount


def checkSltpParam(sltpType=None, slPercent=None, tpPercent=None):
    if enums.SltpOrderType['conditional'] != sltpType and enums.SltpOrderType[
            'oco'] != sltpType:
        return False

    if False == maths.checkPercent(slPercent) or False == maths.checkPercent(
            tpPercent):
        return False

    return True


from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from datetime import datetime


def CrossoverFunc(data0=[], data1=[], period=3):
    '''
    @brief 简易的 crossover判断
    @return  0 条件不足，或者没有满足条件
             1 下穿上
            -1 上穿下
    '''
    retCode = 0
    minLen = min(len(data0), len(data1))
    minLen = min(minLen, period)

    if minLen < period:
        return retCode

    preAllSmall = True
    preAllBig = True
    currBig = False
    currSmall = False

    for i in range(minLen):
        index = -1 * (i + 1)
        if -1 == index:
            if data0[index] > data1[index]:
                currBig = True
            elif data0[index] < data1[index]:
                currSmall = True
        else:
            if data0[index] > data1[index]:
                preAllSmall = False
            elif data0[index] < data1[index]:
                preAllBig = False

        print('i', i, 'currItem@data0', data0[index], 'currItem@data1',
              data1[index])

    if preAllSmall and currBig:
        retCode = 1
    if preAllBig and currSmall:
        retCode = -1

    print('preAllBig', preAllBig, 'preAllSmall', preAllSmall, 'currBig',
          currBig, 'currSmall', currSmall)

    return retCode


from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import argparse


def parse_args():
    parser = argparse.ArgumentParser(description='Multitimeframe test')

    parser.add_argument('--dataname',
                        default='',
                        required=False,
                        help='File Data to Load')

    parser.add_argument('--dataname2',
                        default='',
                        required=False,
                        help='Larger timeframe file to load')

    parser.add_argument('--noresample',
                        action='store_true',
                        help='Do not resample, rather load larger timeframe')

    parser.add_argument('--timeframe',
                        default='weekly',
                        required=False,
                        choices=['daily', 'weekly', 'monthly'],
                        help='Timeframe to resample to')

    parser.add_argument('--compression',
                        default=1,
                        required=False,
                        type=int,
                        help='Compress n bars into 1')

    parser.add_argument('--indicators',
                        action='store_true',
                        help='Wether to apply Strategy with indicators')

    parser.add_argument('--onlydaily',
                        action='store_true',
                        help='Indicator only to be applied to daily timeframe')

    parser.add_argument('--period',
                        default=10,
                        required=False,
                        type=int,
                        help='Period to apply to indicator')

    return parser.parse_args()
